import pytest
from time import sleep

from basestation.scanner import BasestationScanner

"""
It should be kinda obvious, but just FYI, these tests need to be ran near actual hardware for them to succeed.
"""

dev = None


def test_scan():
    global dev
    devices = BasestationScanner.discover()
    assert len(devices) > 0
    dev = devices[0]


def test_connect():
    dev.connect()


def test_identify():
    dev.identify()
    print(
        "\n\nYou should now check if your device flashes 😂. we can't read this state to confirm it is.\n\n"
    )

    sleep(5)


def test_turn_on():
    dev.turn_on()
    assert dev.is_turned_on() is True


def test_turn_off():
    dev.turn_off()
    assert dev.is_turned_on() is False


def test_disconnect():
    dev.disconnect()



SERVICE = "00001523-1212-efde-1523-785feabcd124"
PWR_CHARACTERISTIC = "00001525-1212-EFDE-1523-785FEABCD124"
IDENTIFY_CHARACTERISTIC = "00008421-1212-EFDE-1523-785FEABCD124"
PWR_ON = b"\x01"
PWR_STANDBY = b"\x00"
NAME_SUFFIX = "LHB-"


import argparse
import logging
import sys
from bluepy.btle import Scanner

from .device import BasestationDevice
from .const import NAME_SUFFIX

_logger = logging.getLogger(__name__)


class BasestationScanner:
    def discover():
        devices = []
        scanner = Scanner()
        for d in scanner.scan():
            name = d.getValueText(9)

            if name is not None and name.startswith(NAME_SUFFIX):
                # TODO: should probably also check for existence of service
                devices.append(BasestationDevice(d.addr))

        return devices



from .const import (
    PWR_CHARACTERISTIC,
    IDENTIFY_CHARACTERISTIC,
    PWR_ON,
    PWR_STANDBY,
    SERVICE,
)
from bluepy.btle import Peripheral, ADDR_TYPE_RANDOM


class BasestationDevice:
    def __init__(self, mac):
        self.mac = mac

    def connect(self):
        self.client = Peripheral()
        self.client.connect(self.mac, ADDR_TYPE_RANDOM)
        self.service = self.client.getServiceByUUID(SERVICE)

    def disconnect(self):
        self.client.disconnect()

    def is_turned_on(self):
        char = self.service.getCharacteristics(PWR_CHARACTERISTIC)[0]
        pwr = char.read()
        return pwr != PWR_STANDBY

    def turn_on(self):
        char = self.service.getCharacteristics(PWR_CHARACTERISTIC)[0]
        return char.write(PWR_ON, True)

    def turn_off(self):
        char = self.service.getCharacteristics(PWR_CHARACTERISTIC)[0]
        return char.write(PWR_STANDBY, True)

    def identify(self):
        char = self.service.getCharacteristics(IDENTIFY_CHARACTERISTIC)[0]
        return char.write(
            # any byte will do really, which is why it's not in consts.py
            b"\x01"
        )
